(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "tsparticles-engine", "./Utils"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.BaseMover = void 0;
    const tsparticles_engine_1 = require("tsparticles-engine");
    const Utils_1 = require("./Utils");
    const diffFactor = 2;
    class BaseMover {
        constructor() {
            this._initSpin = (particle) => {
                const container = particle.container, options = particle.options, spinOptions = options.move.spin;
                if (!spinOptions.enable) {
                    return;
                }
                const spinPos = spinOptions.position ?? { x: 50, y: 50 }, spinCenter = {
                    x: (spinPos.x / 100) * container.canvas.size.width,
                    y: (spinPos.y / 100) * container.canvas.size.height,
                }, pos = particle.getPosition(), distance = (0, tsparticles_engine_1.getDistance)(pos, spinCenter), spinAcceleration = (0, tsparticles_engine_1.getRangeValue)(spinOptions.acceleration);
                particle.retina.spinAcceleration = spinAcceleration * container.retina.pixelRatio;
                particle.spin = {
                    center: spinCenter,
                    direction: particle.velocity.x >= 0 ? "clockwise" : "counter-clockwise",
                    angle: particle.velocity.angle,
                    radius: distance,
                    acceleration: particle.retina.spinAcceleration,
                };
            };
        }
        init(particle) {
            const options = particle.options, gravityOptions = options.move.gravity;
            particle.gravity = {
                enable: gravityOptions.enable,
                acceleration: (0, tsparticles_engine_1.getRangeValue)(gravityOptions.acceleration),
                inverse: gravityOptions.inverse,
            };
            this._initSpin(particle);
        }
        isEnabled(particle) {
            return !particle.destroyed && particle.options.move.enable;
        }
        move(particle, delta) {
            const particleOptions = particle.options, moveOptions = particleOptions.move;
            if (!moveOptions.enable) {
                return;
            }
            const container = particle.container, pxRatio = container.retina.pixelRatio, slowFactor = (0, Utils_1.getProximitySpeedFactor)(particle), baseSpeed = (particle.retina.moveSpeed ??= (0, tsparticles_engine_1.getRangeValue)(moveOptions.speed) * pxRatio) *
                container.retina.reduceFactor, moveDrift = (particle.retina.moveDrift ??= (0, tsparticles_engine_1.getRangeValue)(particle.options.move.drift) * pxRatio), maxSize = (0, tsparticles_engine_1.getRangeMax)(particleOptions.size.value) * pxRatio, sizeFactor = moveOptions.size ? particle.getRadius() / maxSize : 1, moveSpeed = (baseSpeed * sizeFactor * slowFactor * (delta.factor || 1)) / diffFactor, maxSpeed = particle.retina.maxSpeed ?? container.retina.maxSpeed;
            if (moveOptions.spin.enable) {
                (0, Utils_1.spin)(particle, moveSpeed);
            }
            else {
                (0, Utils_1.move)(particle, moveOptions, moveSpeed, maxSpeed, moveDrift, delta);
            }
            (0, Utils_1.applyDistance)(particle);
        }
    }
    exports.BaseMover = BaseMover;
});
